/*==================================================================

	CPDImageFilter

		COPYRIGHT (C) 2005 (2006,2007,2009,2010,2013,2018)
		MITSUBISHI ELECTRIC CORPORATION ALL RIGHTS RESERVED

		2005.08.03	Ver.1.0.0
		2005.08.25	Ver.1.1.0
		2006.01.18	Ver.1.2.0
		2006.03.30	Ver.2.0.0
		2007.10.09	Ver.3.0.0
		2009.02.26	Ver.4.0.0
		2010.07.22	Ver.5.0.0
		2010.09.13	Ver.5.0.1
		2010.09.14	Ver.5.0.2
		2013.11.28	Ver.6.0.0
		2018.07.19	Ver.7.0.0

==================================================================*/

#ifndef _CPDImageFilter_h_
#define _CPDImageFilter_h_

#include <Carbon/Carbon.h>

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(8)	/* 8バイト境界 *////*8 bites boundaries*/
#define	IEPANO_DATADIM		3		/* データの次元数(Y,M,C) */
#define	IEPANO_TBLSIZE_GNM	256		/* GNMテーブルサイズ */
	
/*-----  転送データの情報 -----*////*----- Transferred data information -----*/
typedef struct tagCPDBandImageParams {
	void*	baseAddr;	/* イメージデータへのポインタ *////* Pointer to image data */
	long	RowBytes;	/* 1行内のバイト数 *////* Number of bytes in a line */
	Rect	bounds;		/* 転送領域 *////* Transferred area */
} CPDBandImageParams, *PCPDBandImageParams;

typedef struct tagImageEffectParamPano {
	unsigned short	gnm[IEPANO_DATADIM][IEPANO_TBLSIZE_GNM];
} ImageEffectParamPano, *PImageEffectParamPano;

/*----- コントラストテーブルデータ -----*////*----- Contrast table data -----*/
typedef struct tagCPDContrastTable {
    unsigned char    r[256];	/* 赤 *////* Red */
    unsigned char    g[256];	/* 緑 *////* Green */
    unsigned char    b[256];	/* 青 *////* Blue */
} CPDContrastTable, *PCPDContrastTable;

short CP95ChangeImage( CPDBandImageParams* pBandImage,  const CPDContrastTable* pContTbl,
					   double unsRad, double unsAmo, short unsThres );

short CPColorChange( short PrtType, CPDBandImageParams* pBandImage );

/*----- ガンマテーブル -----*/
typedef struct tagCPDGammaTable {
    unsigned short   r[256];  /* the gamma table for red */
    unsigned short   g[256];  /* the gamma table for green */
    unsigned short   b[256];  /* the gamma table for blue */
}CPDGammaTable, *PCPDGammaTable;

/*----- 画像補正パラメータ -----*/
typedef struct tagCPAImageEffectParams {
    unsigned long ver;           /* 構造体のバージョン 104を指定 *////* set the structure version : 104 */
    short    reserved1;          /* reserved */
    short    DLLColorTable;      /* DLL色変換テーブルの指定 *////* Color table for DLL */
    short    reserved2;          /* reserved */
    const CPDContrastTable* pContTbl;    /* コントラストテーブル構造体へのポインタ *////* A pointer which indicates the Contrast table */
    short    sharpness0;         /* シャープネス０ *////* Sharpness 0 */
    const unsigned char* reserved3;	   /* reserved */
    short    reserved4;          /* reserved */
    short    gamma;              /* ガンマ *////* Gamma */
    short    printMode;          /* 印画モードの指定 *////* Printing mode */
    short    reserved6;          /* reserved */
    const CPDGammaTable* pGammaTbl;    /* ガンマテーブル構造体へのポインタ */
    short    overcoatMode;       /* オーバーコート *////* overcoat mode */
    const   CPDGammaTable* pGammaTbl1;   /* Gamma table of Fine mode *////* (support: ver 104 and later) */
    const   CPDGammaTable* pGammaTbl2;   /* Gamma table of SuperFine mode *////* (support: ver 104 and later) */
    const   CPDGammaTable* pGammaTbl3;   /* Gamma table of UltraFine mode *////* (support: ver 104 and later) */
    const   CPDGammaTable* pGammaTbl4;   /* Gamma table of extra SuperFine mode *////* (support: ver 104 and later) */
} CPAImageEffectParams, *PCPAImageEffectParams;

/* ガンマ *////* Gamma */
#define PGamma_1		0x101
#define PGamma_2		0x102
#define PGamma_3		0x103
#define PGamma_4		0x104
#define PGamma_5		0x105

/* 印画モード指定 *////* Print mode designation */
#define PPrintMode_1	0x101
#define PPrintMode_2	0x102
#define PPrintMode_3	0x103
#define PPrintMode_4	0x104
#define PPrintMode_5	0x105
	
/* オーバーコートモード指定 *////* Overcoat mode designation */
#define POverCoat_Standard	0x101
#define POverCoat_Matte		0x102
#define POverCoat_Matte2	0x103
#define POverCoat_StandardL	0x104
#define POverCoat_MatteU	0x105
#define POverCoat_None		0x200
	
	
short CP98ChangeImage( const CPDBandImageParams* pinBandImage, const CPAImageEffectParams*piep,
					   Point printPixel, void* poutBits );

short CP70ChangeImage( const CPDBandImageParams* pinBandImage, const CPAImageEffectParams*piep,
					   Point printPixel, void* poutBits, unsigned char* poutSpeed );

short CP60ChangeImage( const CPDBandImageParams* pinBandImage, const CPAImageEffectParams*piep,
					   Point printPixel, void* poutBits, unsigned char* poutSpeed, unsigned char* prewind );

short CP80ChangeImage( const CPDBandImageParams* pinBandImage, const CPAImageEffectParams*piep,
					   Point printPixel, void* poutBits, unsigned char* poutSpeed );

short CP90PanoramaChangeImage( const CPDBandImageParams* pinBandImage,  const CPAImageEffectParams* piep,
					   Point printPixel, unsigned char *pcombinationNum,
					   short *ppixelY1, short *ppixelY2,
                       void* poutBits1, void* poutBits2, void* poutBits3);

/*-----------------------------------------------------------------------------
	エラーコード//Error Code
-----------------------------------------------------------------------------*/

enum{
Error_NoError = 0,				/* エラー無し *////* No error */
Error_Something,				/* 何らかのエラー(1) *////* Unidentified error */

Error_FuncParamError = 201,     /* DLL関数の引数が不正 *////* Illegal argument of DLL function */
Error_MemAllocError             /* DLLで使用するメモリの確保に失敗した *////* Failed to allocate memory to DLL */
};


#pragma pack()

#ifdef __cplusplus
}
#endif

#endif /* #ifndef _CPDImageFilter_h_ */
