/*==================================================================

	Sample application for CPDImageFilter

        COPYRIGHT (C) 2005 (2006,2007,2009,2010,2013,2018)
        MITSUBISHI ELECTRIC CORPORATION ALL RIGHTS RESERVED

        2018.7.26  Ver.7.0.0.0

==================================================================*/

#include <stdio.h>
#include "CPDImageFilter.h"

int exec_CP95ChangeImage(
	const char* pszImageDataFile, int width, int height,
	const char* pszContrastTableFile,
	double unsRad, double unsAmo, short unsThres,
	const char* pszOutputDataFile );
int load_ContrastTableFile(const char* pszContrastTableFile, CPDContrastTable* pContTbl);
int exec_CPColorChange(
	const char* pszImageDataFile, int width, int height,
	short PrtType,
	const char* pszOutputDataFile );
int save_data( const void* pBuff, int size, const char* pszFileName );
int exec_CP98ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	short printMode,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile);
int prepare_imagedata(const char* pszImageFile, int width, int height, CPDBandImageParams* bandImage);
int exec_CP70ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	const char* pszContrastTableFile,
	short sharpness0,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile);
int exec_CP60ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	const char* pszGammaTableFile,
	short printMode,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile);
int exec_CP80ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	const char* pszGammaTableFile,
	short printMode,
	short printWidth,
	short printHeight,
	short overcoatMode,
	const char* pszGammaTableFile1,
	const char* pszGammaTableFile2,
	const char* pszGammaTableFile3,
	const char* pszGammaTableFile4,
						 const char* pszOutputDataFile);
int exec_CP90PanoramaChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short printMode,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile1,
	const char* pszOutputDataFile2,
	const char* pszOutputDataFile3
);
int ReadGammaTable(const char* pszGammaTableFile, CPDGammaTable* pGammaTbl);

#define MAKEWORD(a, b)      ((unsigned short)(((unsigned char)(a)) | ((unsigned short)((unsigned char)(b))) << 8))

int main (int argc, const char * argv[])
{
	const char* pszImageDataFile = NULL;
	int width, height;
	const char* pszOutputDataFile = "output.raw";
	
    printf("CPDImageFilter Sample Application Start!\n");
	
	if (argc < 5) {
		printf("  Insufficient arguments.\n");
		return 1;
	}
	pszImageDataFile = argv[2];
	width = atoi(argv[3]);
	height = atoi(argv[4]);
	
	if ((0 == strcmp( argv[1], "f1")) || (0 == strcmp( argv[1], "F1"))) {
		/* CP95ChangeImage function */
		const char* pszContrastTableFile = NULL;
		double unsRad = 5.0;
		double unsAmo = 0.5;
		short unsThres = 0;
		
		if (argc < 9) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		if (strcmp(argv[5], "-")) {
			pszContrastTableFile = argv[5];
		}	
		unsRad = atof(argv[6]);
		unsAmo = atof(argv[7]);
		unsThres = atof(argv[8]);

		exec_CP95ChangeImage(pszImageDataFile, width, height, pszContrastTableFile,
							 unsRad, unsAmo, unsThres, pszOutputDataFile);
		
	}
	else if ((0 == strcmp( argv[1], "f2")) || (0 == strcmp( argv[1], "F2"))) {
		/* CPColorChange function */
		short PrtType;
		
		if (argc < 6) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		PrtType = atoi(argv[5]);
		exec_CPColorChange(pszImageDataFile, width, height, PrtType, pszOutputDataFile);
	}
	else if ((0 == strcmp( argv[1], "f3")) || (0 == strcmp( argv[1], "F3"))) {
		/* CP98ChangeImage function */
		short ver = 103;
		short DLLColorTable = 0;
		const char* pszContrastTableFile = NULL;
		short sharpness0  = 6;
		short gamma = 0x101;
		short printMode   = 0x101;
		short printWidth = 1868;
		short printHeight = 1228;
		
		if (argc < 13) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		ver = atoi(argv[5]);
		DLLColorTable  = atoi(argv[6]);
		if (strcmp(argv[7], "-")) {
			pszContrastTableFile = argv[7];
		}
		sharpness0  = atoi(argv[8]);
		gamma  = atoi(argv[9]);
		printMode   = atoi(argv[10]);
		printWidth  = atoi(argv[11]);
		printHeight  = atoi(argv[12]);

		exec_CP98ChangeImage(pszImageDataFile, width, height,
							 ver, DLLColorTable, pszContrastTableFile, sharpness0, gamma, printMode,
							 printWidth, printHeight, pszOutputDataFile);
	}
	else if ((0 == strcmp( argv[1], "f4")) || (0 == strcmp( argv[1], "F4"))) {
		/* CP70ChangeImage function */
		short ver = 103;
		const char* pszContrastTableFile = NULL;
		short sharpness0  = 6;
		short printWidth = 1868;
		short printHeight = 1228;
		
		if (argc < 10) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		ver = atoi(argv[5]);
		if (strcmp(argv[6], "-")) {
			pszContrastTableFile = argv[6];
		}
		sharpness0  = atoi(argv[7]);
		printWidth  = atoi(argv[8]);
		printHeight  = atoi(argv[9]);

		exec_CP70ChangeImage(pszImageDataFile, width, height,
							 ver, pszContrastTableFile, sharpness0,
							 printWidth, printHeight, pszOutputDataFile);
	}
	else if ((0 == strcmp( argv[1], "f5")) || (0 == strcmp( argv[1], "F5"))) {
		/* CP60ChangeImage function */
		short ver = 103;
		short DLLColorTable = -1;
		const char* pszContrastTableFile = NULL;
		short sharpness0  = 6;
		short gamma       = -1;
		const char* pszGammaTableFile = NULL;
		short printMode   = -1;
		short printWidth  = 1868;
		short printHeight = 1218;
		
		if (argc < 14) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		ver = atoi(argv[5]);
		DLLColorTable = atoi(argv[6]);
		if (strcmp(argv[7], "-")) {
			pszContrastTableFile = argv[7];
		}
		sharpness0  = atoi(argv[8]);
		gamma       = atoi(argv[9]);
		if (strcmp(argv[10], "-") != 0) {
			pszGammaTableFile = argv[10];
		}
		printMode   = atoi(argv[11]);
		printWidth  = atoi(argv[12]);
		printHeight = atoi(argv[13]);

		exec_CP60ChangeImage(pszImageDataFile, width, height,
							 ver, DLLColorTable, pszContrastTableFile, sharpness0,
							 gamma, pszGammaTableFile, printMode,
							 printWidth, printHeight, pszOutputDataFile);
	}
	else if ((0 == strcmp( argv[1], "f6")) || (0 == strcmp( argv[1], "F6"))) {
		/* CP80ChangeImage function */
		short ver = 104;
		short DLLColorTable = -1;
		const char* pszContrastTableFile = NULL;
		short sharpness0  = 6;
		short gamma       = -1;
		const char* pszGammaTableFile = NULL;
		short printMode   = -1;
		short printWidth = 1868;
		short printHeight = 1228;
		short overcoatMode = POverCoat_Standard;
		const char* pszGammaTableFile1 = NULL;
		const char* pszGammaTableFile2 = NULL;
		const char* pszGammaTableFile3 = NULL;
		const char* pszGammaTableFile4 = NULL;
		
		if (argc < 19) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		ver = atoi(argv[5]);
		DLLColorTable = atoi(argv[6]);
		if (strcmp(argv[7], "-")) {
			pszContrastTableFile = argv[7];
		}
		sharpness0  = atoi(argv[8]);
		gamma       = atoi(argv[9]);
		if (strcmp(argv[10], "-") != 0) {
			pszGammaTableFile = argv[10];
		}
		printMode   = atoi(argv[11]);
		printWidth  = atoi(argv[12]);
		printHeight = atoi(argv[13]);
		if (strcmp(argv[14], "-") != 0) {
			overcoatMode = atoi(argv[14]);
		}
		if (strcmp(argv[15], "-") != 0) {
			pszGammaTableFile1 = argv[15];
		}
		if (strcmp(argv[16], "-") != 0) {
			pszGammaTableFile2 = argv[16];
		}
		if (strcmp(argv[17], "-") != 0) {
			pszGammaTableFile3 = argv[17];
		}
		if (strcmp(argv[18], "-") != 0) {
			pszGammaTableFile4 = argv[18];
		}

		exec_CP80ChangeImage(pszImageDataFile, width, height,
							 ver, DLLColorTable, pszContrastTableFile, sharpness0,
							 gamma, pszGammaTableFile, printMode,
							 printWidth, printHeight, overcoatMode, 
							 pszGammaTableFile1, pszGammaTableFile2, pszGammaTableFile3, pszGammaTableFile4, pszOutputDataFile);
	}
	else if ((0 == strcmp( argv[1], "f7")) || (0 == strcmp( argv[1], "F7"))) {
		/* CP90PanoramaChangeImage function */
		short ver = 104;
		short DLLColorTable = -1;
		const char* pszContrastTableFile = NULL;
		short printMode   = -1;
		short printWidth = 1868;
		short printHeight = 1228;
		
		if (argc < 11) {
			printf("  Insufficient arguments.\n");
			return 1;
		}
		ver = atoi(argv[5]);
		DLLColorTable = atoi(argv[6]);
		if (strcmp(argv[7], "-")) {
			pszContrastTableFile = argv[7];
		}
		printMode   = atoi(argv[8]);
		printWidth  = atoi(argv[9]);
		printHeight = atoi(argv[10]);
		
		const char* pszOutputDataFileList[3] = {"output1.raw", "output2.raw", "output3.raw"};

		exec_CP90PanoramaChangeImage(pszImageDataFile, width, height,
							 ver, DLLColorTable, pszContrastTableFile,
							 printMode, printWidth, printHeight, 
							 pszOutputDataFileList[0], pszOutputDataFileList[1], pszOutputDataFileList[2]);
	}
	else {
		printf("  Invalid arguments.\n");
		return 1;
	}
	
    printf("CPDImageFilter Sample Application End.\n");
	
    return 0;
}

int exec_CP95ChangeImage(
	const char* pszImageDataFile, int width, int height,
	const char* pszContrastTableFile,
	double unsRad, double unsAmo, short unsThres,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size = 0;
	CPDContrastTable* pContTbl = NULL;
	CPDBandImageParams bip;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)) {
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if ((unsRad < 0.1) || (unsRad > 120.0)) {
		fprintf(stderr, "  Out of range : unsRas(0.1-120.0)\n");
		return 1;
	}
	if ((unsAmo < 0.00) || (unsAmo > 5.00)) {
		fprintf(stderr, "  Out of range : unsAmo(0.00-5.00)\n");
		return 1;
	}
	if ((unsThres < 0) || (unsThres > 255)) {
		fprintf(stderr, "  Out of range : unsThres(0-255)\n");
		return 1;
	}
	if (!pszOutputDataFile) {
		fprintf(stderr, "  Cannot open output data file : \n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		return 1;
	}

	/* load contrast table file */
	if (pszContrastTableFile) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (!pContTbl) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl)) {
			goto EXIT;
		}
	}

	/* call function */
	{
		short errCode = Error_NoError;
	
		fprintf(stderr, "  CPDBandImageParams(baseAddr(%p), bounds(l:%d, t:%d, r:%d, b:%d), RowBytes(%ld)\n",
				bip.baseAddr, bip.bounds.left, bip.bounds.top, bip.bounds.right, bip.bounds.bottom, bip.RowBytes);
		fprintf(stderr, "  pContTbl(%p), unsRad(%lf), unsAmo(%lf), unsThres(%d)\n",
				pContTbl, unsRad, unsAmo, unsThres);
		errCode = CP95ChangeImage(&bip, pContTbl, unsRad, unsAmo, unsThres);
		fprintf(stderr, "  CP95ChangeImage ErrorCode = %d.\n", errCode);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save output data */
	ret = save_data(bip.baseAddr, size, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	if (pContTbl != NULL) {
		free(pContTbl);
	}
	return ret;
}

int load_ContrastTableFile(const char* pszContrastTableFile, CPDContrastTable* pContTbl)
{
	FILE*	infp;
	int		size;

	if ((infp = fopen( pszContrastTableFile, "rb" ) ) == NULL) {
		fprintf(stderr, "  Cannot open the contrast table file : %s.\n", pszContrastTableFile);
		return 1;
	}
	fseek(infp, 0, SEEK_END);
	size = ftell(infp);
	if (size < 256*3) {
		fprintf(stderr, "  Invalid contrast table file : %s.\n", pszContrastTableFile);
		fclose(infp);
		return 1;
	}
	fseek(infp, 0, SEEK_SET);
	fread(pContTbl->r, 1, 256, infp);
	fread(pContTbl->g, 1, 256, infp);
	fread(pContTbl->b, 1, 256, infp);
	fclose(infp);
	return 0;
}


int exec_CPColorChange(
	const char* pszImageDataFile, int width, int height,
	short PrtType,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size;
	CPDContrastTable* pContTbl = NULL;
	CPDBandImageParams bip;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)) {
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if ((PrtType < 0) || (PrtType > 2)) {
		fprintf(stderr, "  Out of range : PrtType(0,2)\n");
		return 1;
	}
	if (!pszOutputDataFile) {
		fprintf(stderr, "  Cannot open output data file : \n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}

	/* call function */
	{
		short errCode = Error_NoError;
	
		fprintf(stderr, "  CPDBandImageParams(baseAddr(%p), bounds(l:%d, t:%d, r:%d, b:%d), RowBytes(%ld)\n",
				bip.baseAddr, bip.bounds.left, bip.bounds.top, bip.bounds.right, bip.bounds.bottom, bip.RowBytes);
		fprintf(stderr, "  PrtType(%d)\n", PrtType);
		errCode = CPColorChange(PrtType, &bip);
		fprintf(stderr, "  CPColorChange ErrorCode = %d.\n", errCode);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save output data */
	ret = save_data(bip.baseAddr, size, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	return ret;
}

int exec_CP98ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	short printMode,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size;
	CPDContrastTable* pContTbl = NULL;
	unsigned char* poutBits = NULL;
	long outputDataSize;
	CPDBandImageParams bip;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)){
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if (ver < 102) {
		fprintf(stderr, "  Out of range : ver(102-)\n");
		return 1;
	}
	if ((DLLColorTable < -1) || (DLLColorTable > 0)) {
		fprintf(stderr, "  Out of range : DLLColorTable(-1,0)\n");
		return 1;
	}
	if ((sharpness0 < -1) || (sharpness0 > 9)) {
		fprintf(stderr, "  Out of range : sharpness0(-1,0-9)\n");
		return 1;
	}
	if ((gamma != -1) && (gamma != 0x101) && (gamma != 0x102)) {
		fprintf(stderr, "  Out of range : gamma(-1,257(0x101)-258(0x102))\n");
		return 1;
	}
	if (ver >= 103) {
		if ((printMode < 0x101) || (printMode > 0x103)) {
			fprintf(stderr, "  Out of range : printMode(257(0x101)-259(0x103))\n");
			return 1;
		}
	}
	if (pszOutputDataFile == NULL) {
		fprintf(stderr, "  Cannot open the output data file : \n");
		return 1;
	}
	if ((printWidth <= 0) || (printHeight <= 0)) {
		fprintf(stderr, "  Invalid printPixel : printWidth, printHeight(1-)\n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}

	/* load a contrast table file */
	if (pszContrastTableFile != NULL) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (pContTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl) != 0) {
			free( pContTbl );
			goto EXIT;
		}
	}
	
	/* allocate output image buffer */
	if ((0 < printWidth) && (0 < printHeight)) {
		outputDataSize = printWidth * printHeight * 3 * sizeof(unsigned short);
		poutBits = (unsigned char*)malloc(outputDataSize);
		if (poutBits == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			ret = 1;
			goto EXIT;
		}
	}

	/* call the function */
	{
		short errCode = Error_NoError;
		CPAImageEffectParams IEParam;
		Point printPixel;
		
		memset(&IEParam, 0, sizeof(IEParam));
		IEParam.ver           = ver;
		IEParam.DLLColorTable = DLLColorTable;
		IEParam.pContTbl      = pContTbl;
		IEParam.sharpness0    = sharpness0;
		IEParam.gamma         = gamma;
		IEParam.printMode     = printMode;
		
		printPixel.h = printWidth;
		printPixel.v = printHeight;
	
		fprintf(stderr, "  CPDBandImageParams(baseAddr(%p), bounds(l:%d, t:%d, r:%d, b:%d), RowBytes(%ld))\n",
				bip.baseAddr, bip.bounds.left, bip.bounds.top, bip.bounds.right, bip.bounds.bottom, bip.RowBytes);
		fprintf(stderr, "  CPAImageEffectParams(ver(%d), DLLColorTable(%d), pContTbl(%p), sharpness0(%d), gamma(%d))\n",
				IEParam.ver, IEParam.DLLColorTable, IEParam.pContTbl, IEParam.sharpness0, IEParam.gamma);
		fprintf(stderr, "  printPixel(h(%d), v(%d))\n",
				printPixel.h, printPixel.v);
		errCode = CP98ChangeImage(&bip, &IEParam, printPixel, poutBits);
		fprintf(stderr, "  CP98ChangeImage ErrorCode = %d.\n", errCode);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save the output data */
	ret = save_data(poutBits, outputDataSize, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	if (pContTbl != NULL) {
		free(pContTbl);
	}
	if (poutBits != NULL) {
		free(poutBits);
	}
	return ret;
}

int exec_CP70ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	const char* pszContrastTableFile,
	short sharpness0,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size = 0;
	CPDContrastTable* pContTbl = NULL;
	unsigned char* poutBits = NULL;
	long outputDataSize = 0;
	CPDBandImageParams bip;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)){
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if (ver < 102) {
		fprintf(stderr, "  Out of range : ver(102-)\n");
		return 1;
	}
	if ((sharpness0 < -1) || (sharpness0 > 8)) {
		fprintf(stderr, "  Out of range : sharpness0(-1,0-8)\n");
		return 1;
	}
	if (pszOutputDataFile == NULL) {
		fprintf(stderr, "  Cannot open the output data file : \n");
		return 1;
	}
	if ((printWidth <= 0) || (printHeight <= 0)) {
		fprintf(stderr, "  Invalid printPixel : printWidth, printHeight(1-)\n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}

	/* load a contrast table file */
	if (pszContrastTableFile != NULL) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (pContTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl) != 0) {
			free( pContTbl );
			goto EXIT;
		}
	}
	
	/* allocate output image buffer */
	if ((0 < printWidth) && (0 < printHeight)) {
		outputDataSize = (printWidth * printHeight * sizeof(unsigned short) + 511) / 512 * 512;
		outputDataSize *= 3;
		poutBits = (unsigned char*)malloc(outputDataSize);
		if (poutBits == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			ret = 1;
			goto EXIT;
		}
	}

	/* call the function */
	{
		short errCode = Error_NoError;
		CPAImageEffectParams IEParam;
		Point printPixel;
		unsigned char speed = 0x00;
		
		memset(&IEParam, 0, sizeof(IEParam));
		IEParam.ver           = ver;
		IEParam.DLLColorTable = -1;
		IEParam.pContTbl      = pContTbl;
		IEParam.sharpness0    = sharpness0;
		IEParam.gamma         = -1;
		IEParam.printMode     = -1;
		
		printPixel.h = printWidth;
		printPixel.v = printHeight;
	
		fprintf(stderr, "  CPDBandImageParams(baseAddr(%p), bounds(l:%d, t:%d, r:%d, b:%d), RowBytes(%ld))\n",
				bip.baseAddr, bip.bounds.left, bip.bounds.top, bip.bounds.right, bip.bounds.bottom, bip.RowBytes);
		fprintf(stderr, "  CPAImageEffectParams(ver(%d), DLLColorTable(%d), pContTbl(%p), sharpness0(%d), gamma(%d))\n",
				IEParam.ver, IEParam.DLLColorTable, IEParam.pContTbl, IEParam.sharpness0, IEParam.gamma);
		fprintf(stderr, "  printPixel(h(%d), v(%d))\n",
				printPixel.h, printPixel.v);
		errCode = CP70ChangeImage(&bip, &IEParam, printPixel, poutBits, &speed);
		fprintf(stderr, "  CP70ChangeImage ErrorCode = %d. speed = %02X\n", errCode, speed);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save the output data */
	ret = save_data(poutBits, outputDataSize, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	if (pContTbl != NULL) {
		free(pContTbl);
	}
	if (poutBits != NULL) {
		free(poutBits);
	}
	return ret;
}

int exec_CP60ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	const char* pszGammaTableFile,
	short printMode,
	short printWidth,
	short printHeight,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size = 0;
	CPDContrastTable* pContTbl = NULL;
	CPDGammaTable* pGammaTbl = NULL;
	unsigned char* poutBits = NULL;
	long outputDataSize = 0;
	CPDBandImageParams bip;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)){
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if (ver < 102) {
		fprintf(stderr, "  Out of range : ver(102-)\n");
		return 1;
	}
	if ((DLLColorTable < -1) || (DLLColorTable > 0)) {
		fprintf(stderr, "  Out of range : DLLColorTable(-1,0)\n");
		return 1;
	}
	if ((sharpness0 < -1) || (sharpness0 > 8)) {
		fprintf(stderr, "  Out of range : sharpness0(-1,0-8)\n");
		return 1;
	}
	if ((gamma != -1) && ((gamma < 0x101) || (gamma > 0x105))) {
		fprintf(stderr, "  Out of range : gamma(-1,257(0x101)-261(0x105))\n");
		return 1;
	}
	if ((printMode != -1) && ((printMode < 0x101) || (printMode > 0x105))) {
		fprintf(stderr, "  Out of range : printMode(-1,257(0x101)-261(0x105))\n");
		return 1;
	}
	if (pszOutputDataFile == NULL) {
		fprintf(stderr, "  Cannot open the output data file : \n");
		return 1;
	}
	if ((printWidth <= 0) || (printHeight <= 0)) {
		fprintf(stderr, "  Invalid printPixel : printWidth, printHeight(1-)\n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}

	/* load a contrast table file */
	if (pszContrastTableFile != NULL) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (pContTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl) != 0) {
			free( pContTbl );
			goto EXIT;
		}
	}
	
	/* load a gamma table file */
	if (pszGammaTableFile != NULL) {
		pGammaTbl = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
		if (pGammaTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (ReadGammaTable(pszGammaTableFile, pGammaTbl) != 0) {
			free( pGammaTbl );
			goto EXIT;
		}
	}
	
	/* allocate output image buffer */
	if ((0 < printWidth) && (0 < printHeight)) {
		outputDataSize = (printWidth * printHeight * sizeof(unsigned short) + 511) / 512 * 512;
		outputDataSize *= 3;
		poutBits = (unsigned char*)malloc(outputDataSize);
		if (poutBits == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			ret = 1;
			goto EXIT;
		}
	}

	/* call the function */
	{
		short errCode = Error_NoError;
		CPAImageEffectParams IEParam;
		Point printPixel;
		unsigned char speed = 0x00;
		unsigned char rollback = 0x00;
		
		memset(&IEParam, 0, sizeof(IEParam));
		IEParam.ver           = ver;
		IEParam.DLLColorTable = DLLColorTable;
		IEParam.pContTbl      = pContTbl;
		IEParam.sharpness0    = sharpness0;
		IEParam.gamma         = gamma;
		IEParam.pGammaTbl     = pGammaTbl;
		IEParam.printMode     = printMode;
		
		printPixel.h = printWidth;
		printPixel.v = printHeight;
	
		fprintf(stderr, "  CPDBandImageParams(baseAddr(%p), bounds(l:%d, t:%d, r:%d, b:%d), RowBytes(%ld))\n",
				bip.baseAddr, bip.bounds.left, bip.bounds.top, bip.bounds.right, bip.bounds.bottom, bip.RowBytes);
		fprintf(stderr, "  CPAImageEffectParams(ver(%d), DLLColorTable(%d), pContTbl(%p), sharpness0(%d), gamma(%d), pGammaTbl(%p), printMode(%d))\n",
				IEParam.ver, IEParam.DLLColorTable, IEParam.pContTbl, IEParam.sharpness0, IEParam.gamma, IEParam.pGammaTbl, IEParam.printMode);
		fprintf(stderr, "  printPixel(h(%d), v(%d))\n",
				printPixel.h, printPixel.v);
		errCode = CP60ChangeImage(&bip, &IEParam, printPixel, poutBits, &speed, &rollback);
		fprintf(stderr, "  CP60ChangeImage ErrorCode = %d. speed = %02X. rollback = %02X\n", errCode, speed, rollback);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save the output data */
	ret = save_data(poutBits, outputDataSize, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	if (pContTbl != NULL) {
		free(pContTbl);
	}
	if (pGammaTbl != NULL) {
		free(pGammaTbl);
	}
	if (poutBits != NULL) {
		free(poutBits);
	}
	return ret;
}

int exec_CP80ChangeImage(
	const char* pszImageDataFile, int width, int height,
	short ver,
	short DLLColorTable,
	const char* pszContrastTableFile,
	short sharpness0,
	short gamma,
	const char* pszGammaTableFile,
	short printMode,
	short printWidth,
	short printHeight,
	short overcoatMode,
	const char* pszGammaTableFile1,
	const char* pszGammaTableFile2,
	const char* pszGammaTableFile3,
	const char* pszGammaTableFile4,
	const char* pszOutputDataFile)
{
	int		ret = 1;
	int		size = 0;
	CPDContrastTable* pContTbl = NULL;
	CPDGammaTable* pGammaTbl = NULL;
	unsigned char* poutBits = NULL;
	long outputDataSize = 0;
	CPDBandImageParams bip;
	CPDGammaTable* pGammaTbl1 = NULL;
	CPDGammaTable* pGammaTbl2 = NULL;
	CPDGammaTable* pGammaTbl3 = NULL;
	CPDGammaTable* pGammaTbl4 = NULL;
	
	memset(&bip, 0, sizeof(bip));

	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)){
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if (ver < 104) {
		fprintf(stderr, "  Out of range : ver(104-)\n");
		return 1;
	}
	if ((DLLColorTable < -1) || (DLLColorTable > 0)) {
		fprintf(stderr, "  Out of range : DLLColorTable(-1,0)\n");
		return 1;
	}
	if ((sharpness0 < -1) || (sharpness0 > 8)) {
		fprintf(stderr, "  Out of range : sharpness0(-1,0-8)\n");
		return 1;
	}
	if ((gamma != -1) && ((gamma < 0x101) || (gamma > 0x105))) {
		fprintf(stderr, "  Out of range : gamma(-1,257(0x101)-261(0x105))\n");
		return 1;
	}
	if ((printMode != -1) && ((printMode < 0x101) || (printMode > 0x105))) {
		fprintf(stderr, "  Out of range : printMode(-1,257(0x101)-261(0x105))\n");
		return 1;
	}
	if (pszOutputDataFile == NULL) {
		fprintf(stderr, "  Cannot open the output data file : \n");
		return 1;
	}
	if ((printWidth <= 0) || (printHeight <= 0)) {
		fprintf(stderr, "  Invalid printPixel : printWidth, printHeight(1-)\n");
		return 1;
	}

	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}

	/* load a contrast table file */
	if (pszContrastTableFile != NULL) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (pContTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl) != 0) {
			goto EXIT;
		}
	}
	/* load a gamma table file */
	if (pszGammaTableFile != NULL) {
		pGammaTbl = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
		if (pGammaTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (ReadGammaTable(pszGammaTableFile, pGammaTbl) != 0) {
			goto EXIT;
		}
	}
	
	if(gamma == PGamma_5){
		
		/* load a gamma1 table file */
		if (pszGammaTableFile1 != NULL) {
			pGammaTbl1 = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
			if (pGammaTbl1 == NULL) {
				fprintf(stderr, "  Memory allocation error.\n");
				goto EXIT;
			}
			if (ReadGammaTable(pszGammaTableFile1, pGammaTbl1) != 0) {
				goto EXIT;
			}
		}
		
		/* load a gamma2 table file */
		if (pszGammaTableFile2 != NULL) {
			pGammaTbl2 = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
			if (pGammaTbl2 == NULL) {
				fprintf(stderr, "  Memory allocation error.\n");
				goto EXIT;
			}
			if (ReadGammaTable(pszGammaTableFile2, pGammaTbl2) != 0) {
				goto EXIT;
			}
		}
		
		/* load a gamma3 table file */
		if (pszGammaTableFile3 != NULL) {
			pGammaTbl3 = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
			if (pGammaTbl3 == NULL) {
				fprintf(stderr, "  Memory allocation error.\n");
				goto EXIT;
			}
			if (ReadGammaTable(pszGammaTableFile3, pGammaTbl3) != 0) {
				goto EXIT;
			}
		}
		
		/* load a gamma4 table file */
		if (pszGammaTableFile4 != NULL) {
			pGammaTbl4 = (CPDGammaTable*)malloc(sizeof(CPDGammaTable));
			if (pGammaTbl4 == NULL) {
				fprintf(stderr, "  Memory allocation error.\n");
				goto EXIT;
			}
			if (ReadGammaTable(pszGammaTableFile4, pGammaTbl4) != 0) {
				goto EXIT;
			}
		}
		
	}
	
	/* allocate output image buffer */
	if ((0 < printWidth) && (0 < printHeight)) {
		outputDataSize = (printWidth * printHeight * sizeof(unsigned short) + 511) / 512 * 512;
		outputDataSize *= 3;
		poutBits = (unsigned char*)malloc(outputDataSize);
		if (poutBits == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			ret = 1;
			goto EXIT;
		}
	}

	/* call the function */
	{
		short errCode = Error_NoError;
		CPAImageEffectParams IEParam;
		Point printPixel;
		unsigned char speed = 0x00;
		
		memset(&IEParam, 0, sizeof(IEParam));
		IEParam.ver           = ver;
		IEParam.DLLColorTable = DLLColorTable;
		IEParam.pContTbl      = pContTbl;
		IEParam.sharpness0    = sharpness0;
		IEParam.gamma         = gamma;
		IEParam.pGammaTbl     = pGammaTbl;
		IEParam.printMode     = printMode;
		IEParam.overcoatMode  = overcoatMode;
		IEParam.pGammaTbl1    = pGammaTbl1;
		IEParam.pGammaTbl2    = pGammaTbl2;
		IEParam.pGammaTbl3    = pGammaTbl3;
		IEParam.pGammaTbl4    = pGammaTbl4;
		
		printPixel.h = printWidth;
		printPixel.v = printHeight;
	
		fprintf(stderr, "  CPAImageEffectParams(ver(%d), DLLColorTable(%d), pContTbl(%p), sharpness0(%d), gamma(%d), pGammaTbl(%p), printMode(%d), overcoatMode(%d), pGammaTbl1(%p), pGammaTbl2(%p), pGammaTbl3(%p), pGammaTbl4(%p))\n",
				IEParam.ver, IEParam.DLLColorTable, IEParam.pContTbl, IEParam.sharpness0, IEParam.gamma, IEParam.pGammaTbl, IEParam.printMode, IEParam.overcoatMode, IEParam.pGammaTbl1, IEParam.pGammaTbl2, IEParam.pGammaTbl3, IEParam.pGammaTbl4);
		fprintf(stderr, "  printPixel(h(%d), v(%d))\n",
				printPixel.h, printPixel.v);
		errCode = CP80ChangeImage(&bip, &IEParam, printPixel, poutBits, &speed);
		fprintf(stderr, "  CP80ChangeImage ErrorCode = %d. speed = %02X\n", errCode, speed);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save the output data */
	ret = save_data(poutBits, outputDataSize, pszOutputDataFile);

EXIT:
	if (bip.baseAddr != NULL) {
		free(bip.baseAddr);
	}
	if (pContTbl != NULL) {
		free(pContTbl);
	}
	if (poutBits != NULL) {
		free(poutBits);
	}
	if(pGammaTbl != NULL){
		free( pGammaTbl );
	}
	if(pGammaTbl1 != NULL){
		free( pGammaTbl1 );
	}
	if(pGammaTbl2 != NULL){
		free( pGammaTbl2 );
	}
	if(pGammaTbl3 != NULL){
		free( pGammaTbl3 );
	}
	if(pGammaTbl4 != NULL){
		free( pGammaTbl4 );
	}
	return ret;
}

int exec_CP90PanoramaChangeImage(
								 const char* pszImageDataFile, int width, int height,
								 short ver,
								 short DLLColorTable,
								 const char* pszContrastTableFile,
								 short printMode,
								 short printWidth,
								 short printHeight,
								 const char* pszOutputDataFile1,
								 const char* pszOutputDataFile2,
								 const char* pszOutputDataFile3
								 ){
	int		ret = 1;
	int		size = 0;
	CPDContrastTable* pContTbl = NULL;
	unsigned char* poutBitsList[3] = {NULL};
	long outputDataSizeList[3] = {0};
	CPDBandImageParams bip;
	CPAImageEffectParams IEParam;
	Point printPixel;
	unsigned char combinationNum = 0;
	short pixelY1 = 0;
	short pixelY2 = 0;
	
	/* initialize */
	memset(&bip, 0, sizeof(bip));
	memset(&IEParam, 0, sizeof(IEParam));
	
	if (!pszImageDataFile) {
		fprintf(stderr, "  Cannot open image data file : \n");
		return 1;
	}
	if ((width <= 0) || (height <= 0)){
		fprintf(stderr, "  Invalid image size : width, height(1-)\n");
		return 1;
	}
	if (ver < 104) {
		fprintf(stderr, "  Out of range : ver(104-)\n");
		return 1;
	}
	if ((DLLColorTable < -1) || (DLLColorTable > 0)) {
		fprintf(stderr, "  Out of range : DLLColorTable(-1,0)\n");
		return 1;
	}
	if ((printMode != -1) && ((printMode < 0x101) || (printMode > 0x105))) {
		fprintf(stderr, "  Out of range : printMode(-1,257(0x101)-261(0x105))\n");
		return 1;
	}
	if (pszOutputDataFile1 == NULL) {
		fprintf(stderr, "  Cannot open the output data file1 : \n");
		return 1;
	}
	if (pszOutputDataFile2 == NULL) {
		fprintf(stderr, "  Cannot open the output data file2 : \n");
		return 1;
	}
	if (pszOutputDataFile3 == NULL) {
		fprintf(stderr, "  Cannot open the output data file3 : \n");
		return 1;
	}
	if ((printWidth <= 0) || (printHeight <= 0)) {
		fprintf(stderr, "  Invalid printPixel : printWidth, printHeight(1-)\n");
		return 1;
	}
	
	/* load image data file */
	if ((size = prepare_imagedata(pszImageDataFile, width, height, &bip)) == 0) {
		goto EXIT;
	}
	
	/* load a contrast table file */
	if (pszContrastTableFile != NULL) {
		pContTbl = (CPDContrastTable*)malloc(sizeof(CPDContrastTable));
		if (pContTbl == NULL) {
			fprintf(stderr, "  Memory allocation error.\n");
			goto EXIT;
		}
		if (load_ContrastTableFile(pszContrastTableFile, pContTbl) != 0) {
			goto EXIT;
		}
	}
	
	/* set parameters */
	{
		IEParam.ver           = ver;
		IEParam.DLLColorTable = DLLColorTable;
		IEParam.pContTbl      = pContTbl;
		IEParam.printMode     = printMode;
		IEParam.gamma         = -1;
		
		printPixel.h = printWidth;
		printPixel.v = printHeight;
	}
	
	/* call the function (calc image num & size)*/
	{
		short errCode = Error_NoError;
		
		/* debug params */
		fprintf(stderr, "  CPAImageEffectParams(ver(%d), DLLColorTable(%d), pContTbl(%p), printMode(%d))\n",
				IEParam.ver, IEParam.DLLColorTable, IEParam.pContTbl, IEParam.printMode);
		fprintf(stderr, "  printPixel(h(%d), v(%d))\n", printPixel.h, printPixel.v);
		
		errCode = CP90PanoramaChangeImage(&bip, &IEParam, printPixel, &combinationNum, &pixelY1, &pixelY2, poutBitsList[0], poutBitsList[1], poutBitsList[2]);
		fprintf(stderr, "  CP90PanoramaChangeImage(1) ErrorCode = %d\n", errCode);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
		fprintf(stderr, "  sizes (combinationNum(%d), pixelY1(%d), pixelY2(%d))\n", combinationNum, pixelY1, pixelY2);
	}
	
	/* allocate output image buffer */
	if (0 < printWidth) {
		int i;
		for(i=1; i<=combinationNum; i++){
			int idx = i-1;
			long printHeight = 0;
			
			/* define height */
			if(i==combinationNum){
				printHeight = pixelY2;
			}
			else{
				printHeight = pixelY1;
			}
			if(printHeight <= 0) continue;
			
			/* allocate memory */
			//outputDataSizeList[idx] = (printWidth * printHeight * sizeof(unsigned short) + 511) / 512 * 512;
			outputDataSizeList[idx] = printWidth * printHeight * sizeof(unsigned short);
			outputDataSizeList[idx] *= 3;
			if(outputDataSizeList[idx] != 0){
				poutBitsList[idx] = (unsigned char*)malloc(outputDataSizeList[idx]);
				if (poutBitsList == NULL) {
					fprintf(stderr, "  Memory allocation error.\n");
					ret = 1;
					goto EXIT;
				}
			}
		}
	}
	
	/* call the function */
	{
		short errCode = Error_NoError;
		errCode = CP90PanoramaChangeImage(&bip, &IEParam, printPixel, &combinationNum, &pixelY1, &pixelY2, poutBitsList[0], poutBitsList[1], poutBitsList[2]);
		fprintf(stderr, "  CP90PanoramaChangeImage(2) ErrorCode = %d\n", errCode);
		if (errCode != Error_NoError) {
			goto EXIT;
		}
	}
	
	/* save the output data */
	{
		const char* pszOutputDataFileList[3] = {pszOutputDataFile1, pszOutputDataFile2, pszOutputDataFile3};
		int i;
		for(i=0; i<combinationNum; i++){
			ret = save_data(poutBitsList[i], outputDataSizeList[i], pszOutputDataFileList[i]);
			if(ret != 0) goto EXIT;
		}
	}
	
EXIT:
	{
		int i;
		if (bip.baseAddr != NULL) {
			free(bip.baseAddr);
		}
		if (pContTbl != NULL) {
			free(pContTbl);
		}
		for(i=0; i<3; i++){
			if (poutBitsList[i] != NULL) {
				free(poutBitsList[i]);
			}
		}
	}
	return ret;
}

int save_data(const void* pBuff, int size, const char* pszFileName)
{
	FILE*	outfp;
	int		wsize;
	if ((outfp = fopen(pszFileName, "wb")) == NULL) {
		fprintf(stderr, "  Cannot open the output file \n", pszFileName);
		return 1;
	}
	wsize = fwrite(pBuff, 1, size, outfp);
	fclose(outfp);
	
	return 0;
}

int prepare_imagedata(const char* pszImageFile, int width, int height, CPDBandImageParams* bandImage)
{
	FILE*	infp = NULL;
	int		size;
	int		rSize;
	unsigned char* pBuff = 0;

	if ((infp = fopen(pszImageFile, "rb")) == NULL) {
		fprintf(stderr, "  Cannot open the image data file : %s.\n", pszImageFile);
		goto EXIT;
	}
	fseek(infp, 0, SEEK_END);
	size = ftell(infp);
	pBuff = malloc(size);
	if (!pBuff) {
		fprintf(stderr, "  Memory allocation error.\n");
		goto EXIT;
	}
	fseek(infp, 0, SEEK_SET);
	rSize = fread(pBuff, 1, size, infp);
	fclose(infp);
	infp = NULL;
	
	bandImage->bounds.left   = 0;
	bandImage->bounds.top    = 0;
	bandImage->bounds.right  = width;
	bandImage->bounds.bottom = height;
	bandImage->RowBytes      = width * 3;
	bandImage->baseAddr      = pBuff;
	
	return size;
	
EXIT:	/* Error exit */
	if (infp != NULL) {
		free(infp);
	}
	if (pBuff != NULL) {
		free(pBuff);
	}
	return 0;
}

int ReadGammaTable(const char* pszGammaTableFile, CPDGammaTable* pGammaTbl)
{

	if (strlen(pszGammaTableFile) > 0) {

		FILE* rf = NULL;
		if ( (rf = fopen(pszGammaTableFile, "rb")) == NULL ) {
			fprintf(stderr, "  Cannot open the gamma table file : %s.\n", pszGammaTableFile);
			return 1;
		}

		memset(pGammaTbl, 0x00, sizeof(CPDGammaTable));

		fseek(rf, 0, SEEK_END);
		if ( (16 + 256 * 2 * 3) > ftell(rf) ) {
			fclose(rf);
			fprintf(stderr, "  Invalid gamma table file : %s.\n", pszGammaTableFile);
			return 1;
		}

		fseek(rf, 16, SEEK_SET);
		{
			char buf[512];
			int i;
			// R
			memset(buf, 0, sizeof(buf));
			fread(buf, 1, 512, rf);
			for (i=0; i<256; i++) {
				pGammaTbl->r[i] = MAKEWORD(buf[i*2], buf[i*2+1]);
			}
			// G
			memset(buf, 0, sizeof(buf));
			fread(buf, 1, 512, rf);
			for (i=0; i<256; i++) {
				pGammaTbl->g[i] = MAKEWORD(buf[i*2], buf[i*2+1]);
			}
			// B
			memset(buf, 0, sizeof(buf));
			fread(buf, 1, 512, rf);
			for (i=0; i<256; i++) {
				pGammaTbl->b[i] = MAKEWORD(buf[i*2], buf[i*2+1]);
			}
			
		}

		fclose(rf);
	}

	return 0;
}
